/* Program09_3
 *
 * Csoportos adatküldés egy DS32231 RTC-nek.
 * A kiküldött adatokkal feltöltjük a DS32231 RTC
 * regisztereit, beállítva ezzel az üzemmódot és
 * az induló időt/dátumot. 
 *
 * Az I2C kapcsolathoz az I2C1 csatornát és a 
 * PB8 (SCL), PB9 (SDA) kivezetéseket használjuk.
 * A kommunikáció Fast módban (400 kHz), 2:1
 * kitöltéssel (DUTY = 0) megy.
 *
 * Kapcsolás:
 *   --------           -------------
 *  | STM32  |         | DS3231 RTC  | 
 *  |    PB9 |---------| SDA         |
 *  |    PB8 |---------| SCL         |  
 *  |     5V |---------| VCC         |
 *  |    GND |---------| GND         |
 *   --------           -------------
 * Felhasznált forrás: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems    
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"
#define SLAVE_ADDR 0x68    /* 1101 000.    DS3231 */

void I2C1_init(void);
void I2C1_burstWrite(char saddr, char maddr, int size, char* data);

int main(void) {
  /*                          00    01    02    03    04    05    06 */
  char timeDateToSet[15] = {0x55, 0x56, 0x11, 0x02, 0x28, 0x01, 0x20, 0};
                              /* 2020. Január 28, Kedd, 11:58:55 */
  I2C1_init();
  I2C1_burstWrite(SLAVE_ADDR, 0, 7, timeDateToSet);
  
  while (1) {
  }
}

void I2C1_init() {
  RCC->APB1ENR |=  RCC_APB1ENR_I2C1EN;        // I2C1 engedélyezése
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;         // AFIO engedélyezése	
  RCC->APB2ENR |=  RCC_APB2ENR_IOPBEN;        // GPIOB engedélyezése
  AFIO->MAPR |= AFIO_MAPR_I2C1_REMAP;         // I2C1 REMAP=1 (SCL/PB8, SDA/PB9)
  GPIOB->CRH |= GPIO_CRH_CNF8|GPIO_CRH_MODE8; // PB8 ALT/OD kimenet (SCL)
  GPIOB->CRH |= GPIO_CRH_CNF9|GPIO_CRH_MODE9; // PB9 ALT/OD kimenet (SDA) 
  I2C1->CR1 =   I2C_CR1_SWRST;                // Szoftver RESET
  I2C1->CR1 &= ~I2C_CR1_SWRST;                // RESET vége
  /* FREQ = PCLK1/1_000_000         */ 
  I2C1->CR2 = 36;                             // PCLK1 = 36 MHz
  /* CCR = PCLK1/ClkFastSpeed/3, FS =1, DUTY = 0  */
  I2C1->CCR = I2C_CCR_FS + 30;                // Fast mode 400 kHz, 2:1 duty
  /* TRISE = tRISE/tPCLK1+1 = 300ns*36 MHz +1 */
  I2C1->TRISE = 11;                           // maximum felfutási ido
  I2C1->CR1 |=  I2C_CR1_PE;                   // I2C1 modul indítása
}

void I2C1_burstWrite(char saddr, char maddr, int size, char* data)  {
  volatile int i, tmp;
  while (I2C1->SR2 & 2);                      /* Vár, amíg a busz foglalt */
  I2C1->CR1 |=  0x100;                        /* START generálás */
  while (!(I2C1->SR1 & 1));                   /* Vár, amíg SR1.SB = 0 */
  I2C1->DR = saddr<<1;                        /* Slave cím kiküldése */
  while (!(I2C1->SR1 & 2));                   /* Vár, amíg SR1.ADDR = 0 */
  tmp = I2C1->SR2;                            /* SR1.ADDR törlése */
  while (!(I2C1->SR1 & 0x80));                /* Vár, amíg SR1.TXE = 0 */
  I2C1->DR = maddr;                           /* regisztercím küldése */
  for(i = 0; i < size; i++) {
    while (!(I2C1->SR1 & 0x80));              /* Vár, amíg SR1.TXE = 0 */
    I2C1->DR = data[i];                       /* Adatbájt küldése */
  }
  while (!(I2C1->SR1 & 4));                   /* Vár, amíg SR1.BTF = 0  */
  I2C1->CR1 |= 0x200;                         /* STOP generálás */
}



