/* Program09_4
 *
 * Csoportos adatbeolvasás egy DS32231 RTC-ről.
 * A beolvasott adatok az RTC-ből kiolvasott
 * aktuális időt és dátumot adják meg.
 * Az alábbi programban az időt debug módban
 * az SWO kimenetre átirányított printf()
 * segítségével kiírjuk.
 *
 * Az I2C kapcsolathoz az I2C1 csatornát és a 
 * PB8 (SCL), PB9 (SDA) kivezetéseket használjuk.
 * A kommunikáció standard módban (100 kHz), 1:1
 * kitöltéssel (F/S = 0, DUTY = 0) megy.
 *
 * Kapcsolás:
 *   --------           -------------
 *  | STM32  |         | DS3231 RTC  | 
 *  |    PB9 |---------| SDA         |
 *  |    PB8 |---------| SCL         |  
 *  |     5V |---------| VCC         |
 *  |    GND |---------| GND         |
 *   --------           -------------
 * Felhasznált forrás: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems    
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"
//----- printf átirányítás ------------
#include <stdio.h>
#pragma import(__use_no_semihosting_swi)
struct __FILE { int handle; };
FILE __stdout;
FILE __stdin;
int fputc(int ch, FILE *f) {
ITM_SendChar(ch);
return (ch);
}
void _sys_exit(int return_code) {
label: goto label; /* endless loop */
}
//--- printf átírányítás vége ---------

#define SLAVE_ADDR 0x68    /* 1101 000.    DS3231 */
void I2C1_init(void);
void I2C1_burstRead(char saddr, char maddr, int n, char* data);
void displayTime(char* data); 

int main(void) {
  char timeDateReadback[15];
  char lasttime =0;
  I2C1_init();

  while (1) {
    I2C1_burstRead(SLAVE_ADDR, 0, 7, timeDateReadback);
    if (timeDateReadback[0] != lasttime) {
      displayTime(timeDateReadback);
      lasttime = timeDateReadback[0];
    }
  }
}


void I2C1_init(void) {
  RCC->APB1ENR |=  RCC_APB1ENR_I2C1EN;	      // I2C1 engedélyezése
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;         // AFIO engedélyezése	
  RCC->APB2ENR |=  RCC_APB2ENR_IOPBEN;        // GPIOB engedélyezése
  AFIO->MAPR |= AFIO_MAPR_I2C1_REMAP;         // I2C1 REMAP=1 (SCL/PB8, SDA/PB9)
  GPIOB->CRH |= GPIO_CRH_CNF8|GPIO_CRH_MODE8; // PB8 ALT/OD kimenet (SCL)
  GPIOB->CRH |= GPIO_CRH_CNF9|GPIO_CRH_MODE9; // PB9 ALT/OD kimenet (SDA) 
  I2C1->CR1 =   I2C_CR1_SWRST;                // Szoftver RESET
  I2C1->CR1 &= ~I2C_CR1_SWRST;                // RESET vége
  /* FREQ = PCLK1/1_000_000         */ 
  I2C1->CR2 = 36;                             // PCLK1 = 36 MHz
  /* CCR = max(PCLK1/ClkSpeed/2,4)  */
  I2C1->CCR = 180;                            // Standard mód 100 kHz
  /* TRISE = PCLK1/1000000+1        */
  I2C1->TRISE = 37;                           // maximum felfutási idő
  I2C1->CR1 |=  I2C_CR1_PE;                   // I2C1 modul indítása
}

void I2C1_burstRead(char saddr, char maddr, int size, char* data)  {
  volatile int tmp;
  while (I2C1->SR2 & 2);                  /* wait until bus not busy */
  I2C1->CR1 &= ~0x800;                    /* disable POS */
  I2C1->CR1 |= 0x100;                     /* generate start */
  while (!(I2C1->SR1 & 1));               /* wait until start flag is set */
  I2C1->DR = saddr << 1;                  /* transmit slave address + Write */
  while (!(I2C1->SR1 & 2));               /* wait until addr flag is set */
  tmp = I2C1->SR2;                        /* clear addr flag */
  while (!(I2C1->SR1 & 0x80));            /* wait until transmitter empty */
  I2C1->DR = maddr;                       /* send memory address */
  while (!(I2C1->SR1 & 0x80));            /* wait until transmitter empty */

  I2C1->CR1 |= 0x100;                     /* generate restart */
  while (!(I2C1->SR1 & 1));               /* wait until start flag is set */
  I2C1->DR = saddr << 1 | 1;              /* transmit slave address + Read */
  while (!(I2C1->SR1 & 2));               /* wait until addr flag is set */
  tmp = I2C1->SR2;                        /* clear addr flag */ 
  I2C1->CR1 |=  0x0400;                   /* Enable Acknowledge */
  while(size > 0U) {
    /* One byte left */
    if(size == 1U) {
      I2C1->CR1 &= ~(0x400);              /* Disable Acknowledge */
      I2C1->CR1 |= 0x200;                 /* Generate Stop */
      while (!(I2C1->SR1 & 0x40));        /* Wait for RXNE flag set */
      *data++ = I2C1->DR;                 /* Read data from DR */
      break;
    }
    else {
      while (!(I2C1->SR1 & 0x40));        /* Wait until RXNE flag is set */
      (*data++) = I2C1->DR;               /* Read data from DR */
      size--;
    }
  }
}

#define dec1(x)     (x &0x0f)+48
#define dec10(x)    ((x>>4)&0x03)+48 

void displayTime(char* data) {
  char ts[9];
  ts[8]=0;
  ts[7]=dec1(data[0]);
  ts[6]=dec10(data[0]);
  ts[5]=':';
  ts[4]=dec1(data[1]);
  ts[3]=dec10(data[1]);
  ts[2]=':';
  ts[1]=dec1(data[2]);
  ts[0]=dec10(data[2]);	
  printf("%s\n",ts);
}



