/* Program09_6
 *
 * Ez a program az I2C1 buszon keresztül kezeli 
 * az SH1106 vezérlővel ellátott OLED kijelzőt.
 *
 * Az SH1106 vezérlő nem teljesen kompatibilis az SSD1306-tal:
 * - Az oszlopokat 2-vel el kell tolni (hogy a RAM cím 2-129 legyen)
 * - Nincsenek scroll (görgető) parancsok
 * - Apróbb különbségek lehetnek az inicializálásban
 * Az ebben a projetben használt SSD0306.c forrásfájl már tartalmazza 
 * az oszlopok eltolását és kihagytuk belőle a görgető parancsokat.
 * Az I2C kapcsolat fixen PB8, PB9-hez rendelt, 400 kHz sebességű.
 *
 * Kapcsolás:
 *   --------           --------------
 *  | STM32  |         | SH1106 OLED |
 *  |    PB9 |---------| SDA          |
 *  |    PB8 |---------| SCL          | 
 *  |    GND |---------| GND          |
 *  |    VCC |---------| VCC          |
 *   --------           --------------
 *
 * Felhasznált források: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems    
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Controllerstech: OLED display using I2C and STM32
 * https://controllerstech.com/oled-display-using-i2c-stm32/
 *
 * Alexander Lutsai: Блиблиотека OLED дисплея SSD1306 для STM32 микроконтроллеров
 * https://lutsai.ru/stm32/2016/03/08/Library-ssd1306-stm32.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"
#include "ssd1306.h"

int main(void) {
  SSD1306_Init();            // inicializálás
  //--- Írjunk ki valamit! -------
  SSD1306_GotoXY(5, 5);
  SSD1306_Puts("HELLO", &Font_11x18, SSD1306_COLOR_WHITE);
  SSD1306_GotoXY(10, 35);
  SSD1306_Puts("  WORLD :)", &Font_11x18, SSD1306_COLOR_WHITE);
  //--- Keretező téglalap kirajzolása ------
  SSD1306_DrawRectangle(0,0,127,63,SSD1306_COLOR_WHITE);
  SSD1306_UpdateScreen();    // Megjelenítés
  while(1) { }
}







